package cn.com.qmth.examcloud.service.examwork.api;

import io.swagger.annotations.ApiOperation;

import java.io.IOException;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import cn.com.qmth.examcloud.common.util.ErrorMsg;
import cn.com.qmth.examcloud.common.util.excel.ExcelError;
import cn.com.qmth.examcloud.service.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.service.examwork.entity.ExamStudent;
import cn.com.qmth.examcloud.service.examwork.service.ExamStudentService;

/**
 * 考生服务API
 * Created by songyue on 17/1/13.
 */
@RestController
@RequestMapping("${app.api.root}")
public class ExamStudentApi {

    @Autowired
    ExamStudentRepo examStudentRepo;

    @Autowired
    ExamStudentService examStudentService;

    @ApiOperation(value="查询所有考试学生",notes = "分页")
    @GetMapping("/exam_student/all/{curPage}/{pageSize}")
    public ResponseEntity getAllExamStudent(@ModelAttribute ExamStudent examCriteria, @PathVariable Integer curPage, @PathVariable Integer pageSize){
        return new ResponseEntity(examStudentService.getAllExamStudent(examCriteria,new PageRequest(curPage - 1,pageSize)),HttpStatus.OK);
    }

    @ApiOperation(value="查询所有考试学生",notes = "不分页")
    @GetMapping("/exam_student/all")
    public ResponseEntity getAllExamStudent(@ModelAttribute ExamStudent examCriteria){
        return new ResponseEntity(examStudentService.getAllExamStudent(examCriteria), HttpStatus.OK);
    }

    @ApiOperation(value="查询所有考试学生",notes = "不分页不带查询条件")
    @GetMapping("/exam_student")
    public ResponseEntity getAllExamStudent(){
        return new ResponseEntity(examStudentRepo.findAll(), HttpStatus.OK);
    }

    @ApiOperation(value="按ID查询考试学生",notes = "ID查询")
    @GetMapping("/exam_student/{id}")
    public ResponseEntity<ExamStudent> getExamStudentById(@PathVariable Long id){
        return new ResponseEntity<ExamStudent>(examStudentService.getExamStudentById(id),HttpStatus.OK);
    }

    @ApiOperation(value="新增考试学生",notes = "新增")
    @PostMapping("/exam_student")
    public ResponseEntity addExamStudent(@RequestBody ExamStudent examStudent){
        return new ResponseEntity<ExamStudent>(examStudentService.saveExamStudent(examStudent),HttpStatus.OK);
    }

    @ApiOperation(value="更新考试学生",notes = "更新")
    @PutMapping("/exam_student")
    public ResponseEntity updateExamStudent(@RequestBody ExamStudent examStudent){
        return new ResponseEntity(examStudentService.saveExamStudent(examStudent),HttpStatus.OK);
    }

    @ApiOperation(value="按ID删除考试学生",notes = "删除")
    @DeleteMapping("/exam_student/{id}")
    public ResponseEntity deleteExamStudent(@PathVariable Long id){
    	examStudentService.deleteExamStudent(id);
        return new ResponseEntity(HttpStatus.OK);
    }
    
    @ApiOperation(value="导入考试学生",notes = "导入")
    @PostMapping("/exam_student/import")
    public ResponseEntity importExamStudent(@RequestParam Long examId,@RequestParam MultipartFile file){
    	try {
    		List<ExcelError> excelErrors = examStudentService.importExamStudent(examId,file.getInputStream());
    		return new ResponseEntity(excelErrors,HttpStatus.OK);
		} catch (IOException e) {
			e.printStackTrace();
			return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
		}
    }
    
//    @ApiOperation(value="导入学生照片",notes = "导入")
//    @PostMapping("/exam_student/import_photo")
//    public ResponseEntity importPhoto(@RequestParam Long examId,@RequestParam MultipartFile file){
//    	try {
//    		List<ErrorMsg> errorMsgs = examStudentService.importPhoto(examId,file.getInputStream());
//    		return new ResponseEntity(errorMsgs,HttpStatus.OK);
//		} catch (IOException e) {
//			e.printStackTrace();
//			return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
//		}
//    }
    
    @ApiOperation(value="照片检验",notes = "检验")
    @PostMapping("/exam_student/photo_check")
    public ResponseEntity photoCheck(@RequestParam Long examId){
    	try {
    		List<ErrorMsg> errorMsgs = examStudentService.photoCheck(examId);
    		return new ResponseEntity(errorMsgs,HttpStatus.OK);
		} catch (Exception e) {
			e.printStackTrace();
			return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
		}
    }
}
