package cn.com.qmth.examcloud.service.examwork.service;

import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.startsWith;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.criteria.Predicate;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import cn.com.qmth.examcloud.common.util.ErrorMsg;
import cn.com.qmth.examcloud.common.util.excel.ExcelError;
import cn.com.qmth.examcloud.common.util.excel.ExcelReader;
import cn.com.qmth.examcloud.common.util.excel.ExcelReaderHandle;
import cn.com.qmth.examcloud.service.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.service.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.service.examwork.entity.Exam;
import cn.com.qmth.examcloud.service.examwork.entity.ExamStudent;

/**
 * 考试学生服务类
 * Created by songyue on 17/1/14.
 */
@Service
public class ExamStudentService {
    @Autowired
    ExamStudentRepo examStudentRepo;
    @Autowired
    ExamRepo examRepo;
    
//    @Value("${app.em.photo.path}")
//    private String PHOTO_PATH ;
//    @Value("${app.em.facepp.key}")
//    private String FACEPP_KEY;
//    @Value("${app.em.facepp.secret}")
//    private String FACEPP_SECRET;
    
    /**
     * 获取所有考试学生(分页)
     * @param examCriteria
     * @param pageable
     * @return 
     * @return
     */
    public  Page<ExamStudent> getAllExamStudent(ExamStudent examCriteria, Pageable pageable){
//        ExampleMatcher exampleMatcher = ExampleMatcher.matching()
//                .withMatcher("name",startsWith());
//        Example<ExamStudent> examExamStudentple = Example.of(examCriteria, exampleMatcher);
//        return examStudentRepo.findAll(examExamStudentple,pageable);
    	Specification<ExamStudent> specification = (root, query, cb) -> {
            List<Predicate> predicates = new ArrayList<>();
            predicates.add(cb.like(root.get("name"),examCriteria.getName()));
            predicates.add(cb.like(root.get("studentCode"),examCriteria.getStudentCode()));
            return cb.and(predicates.toArray(new Predicate[predicates.size()]));
        };
        return examStudentRepo.findAll(specification,pageable);
    }

    /**
     * 获取所有考试学生
     * @param examCriteria
     * @return 
     * @return
     */
    public  List<ExamStudent> getAllExamStudent(ExamStudent examCriteria){
        ExampleMatcher exampleMatcher = ExampleMatcher.matching()
                .withMatcher("name",startsWith());
        Example<ExamStudent> examExamStudentple = Example.of(examCriteria, exampleMatcher);
        return examStudentRepo.findAll(examExamStudentple);
    }

    /**
     * 按ID获取考试学生
     * @param studentId
     * @return
     */
    public ExamStudent getExamStudentById(Long studentId){
        return examStudentRepo.findOne(studentId);
    }

    /**
     * 按考试批次获取学生
     * @param examId
     * @return 
     * @return
     */
    public Page<ExamStudent> getExamStudentByExamId(Long examId,Pageable pageable){
        return examStudentRepo.findByExamId(examId,pageable);
    }

    /**
     * 保存考试学生
     * @param exam
     * @return
     */
    public ExamStudent saveExamStudent(ExamStudent exam){
        return examStudentRepo.save(exam);
    }

    /**
     * 删除考试学生
     * @param examId
     * @return
     */
    public void deleteExamStudent(Long examId){
        examStudentRepo.delete(examId);
    }

    /**
     * 删除所有考试学生
     * @return
     */
    public void deleteAllExamStudent(){
        examStudentRepo.deleteAll();
    }
    
    /**
     * 导入考试学生
     * @return
     */
	public List<ExcelError> importExamStudent(Long examId, InputStream inputStream) {
		Exam exam = examRepo.findOne(examId);
		List<ExamStudent> examStudents = new ArrayList<ExamStudent>();
		ExcelReader excelReader = new ExcelReader(ExamStudent.class);
        List<ExcelError> excelErrors = excelReader.reader(inputStream, new ExcelReaderHandle() {
            @Override
            public ExcelError handle(Object obj) {
                ExamStudent dto = (ExamStudent) obj;
                dto.setRootOrgId(exam.getOrgId());
                dto.setExam(exam);
                ExcelError error = importCheck(dto);
                if (error == null) {
                	examStudents.add(dto);
                }
                if (examStudents.size() % 1000 == 0) {
                    examStudentRepo.save(examStudents);
                    examStudents.removeAll(examStudents);
                }
                return error;
            }
        });
        examStudentRepo.save(examStudents);
        examStudents.removeAll(examStudents);
        return excelErrors;
	}
	
	/**
	 * 考生导入验证
	 * @param stu
	 * @return
	 */
	public ExcelError importCheck(ExamStudent dto){
		
		return null;
	}
	
	/**
	 * 照片检验
	 * @param examId
	 * @return
	 */
	public List<ErrorMsg> photoCheck(Long examId) {
		List<ErrorMsg> msgs = new ArrayList<ErrorMsg>();
		List<ExamStudent> examStudents = examStudentRepo.findByExamId(examId);
		for (ExamStudent examStudent : examStudents) {
			ErrorMsg errorMsg = null;
			if(!StringUtils.isEmpty(examStudent.getStudentCode())){
				
			}else if(!StringUtils.isEmpty(examStudent.getIdentityNumber())){
				
			}
		}
		return msgs;
	}

}
